package com.bipt.intelligentapplicationorchestrationservice.util;

import com.aliyun.oss.HttpMethod;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.*;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.io.*;
import java.net.URL;
import java.util.Date;

/**
 * 阿里云OSS文件操作工具类
 */
@Data
@Slf4j
@AllArgsConstructor
public class AliOssUtil {

    private final String endpoint;
    private final String accessKeyId;
    private final String accessKeySecret;
    private final String bucketName;

    /**
     * 上传文件到OSS
     * @param file 文件对象
     * @param objectName 对象名称（OSS中的路径）
     * @return 文件URL
     */
    public String upload(File file, String objectName) {
        try (InputStream inputStream = new FileInputStream(file)) {
            return upload(inputStream, objectName);
        } catch (Exception e) {
            log.error("上传文件失败: {}", e.getMessage(), e);
            throw new RuntimeException("上传文件到OSS失败", e);
        }
    }

    /**
     * 上传文件流到OSS
     * @param inputStream 文件流
     * @param objectName 对象名称（OSS中的路径）
     * @return 文件URL
     */
    public String upload(InputStream inputStream, String objectName) {
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        try {
            // 创建PutObject请求
            PutObjectRequest request = new PutObjectRequest(bucketName, objectName, inputStream);

            // 设置对象元数据
            ObjectMetadata metadata = new ObjectMetadata();
            metadata.setContentType(getContentType(objectName));
            request.setMetadata(metadata);

            // 上传文件
            ossClient.putObject(request);
            log.info("文件上传成功: {}", objectName);

            // 构建文件URL
            return "https://" + bucketName + "." + endpoint + "/" + objectName;
        } catch (Exception e) {
            log.error("上传文件失败: {}", e.getMessage(), e);
            throw new RuntimeException("上传文件到OSS失败", e);
        } finally {
            // 关闭OSSClient
            ossClient.shutdown();
        }
    }

    /**
     * 生成临时签名URL，用于访问私有Bucket中的文件
     * @param objectName 对象名称
     * @param expirationMinutes 过期时间（分钟）
     * @return 签名URL
     */
    public String generatePresignedUrl(String objectName, int expirationMinutes) {
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        try {
            // 设置URL过期时间
            Date expiration = new Date(System.currentTimeMillis() + expirationMinutes * 60 * 1000);

            // 生成签名URL
            GeneratePresignedUrlRequest request = new GeneratePresignedUrlRequest(bucketName, objectName, HttpMethod.GET);
            request.setExpiration(expiration);

            URL url = ossClient.generatePresignedUrl(request);
            log.info("生成临时签名URL: {}", url);
            return url.toString();
        } catch (Exception e) {
            log.error("生成签名URL失败: {}", e.getMessage(), e);
            throw new RuntimeException("生成签名URL失败", e);
        } finally {
            // 关闭OSSClient
            ossClient.shutdown();
        }
    }

    /**
     * 下载文件到本地
     * @param objectName 对象名称
     * @param destinationFile 目标文件
     */
    public void download(String objectName, File destinationFile) {
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        try {
            // 下载OSS文件到本地文件
            ossClient.getObject(new GetObjectRequest(bucketName, objectName), destinationFile);
            log.info("文件下载成功: {}", objectName);
        } catch (Exception e) {
            log.error("下载文件失败: {}", e.getMessage(), e);
            throw new RuntimeException("下载文件失败", e);
        } finally {
            // 关闭OSSClient
            ossClient.shutdown();
        }
    }

    /**
     * 删除OSS中的文件
     * @param objectName 对象名称
     */
    public void delete(String objectName) {
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        try {
            // 删除文件
            ossClient.deleteObject(bucketName, objectName);
            log.info("文件删除成功: {}", objectName);
        } catch (Exception e) {
            log.error("删除文件失败: {}", e.getMessage(), e);
            throw new RuntimeException("删除文件失败", e);
        } finally {
            // 关闭OSSClient
            ossClient.shutdown();
        }
    }

    /**
     * 检查文件是否存在
     * @param objectName 对象名称
     * @return 文件是否存在
     */
    public boolean doesObjectExist(String objectName) {
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        try {
            return ossClient.doesObjectExist(bucketName, objectName);
        } catch (Exception e) {
            log.error("检查文件存在失败: {}", e.getMessage(), e);
            throw new RuntimeException("检查文件存在失败", e);
        } finally {
            // 关闭OSSClient
            ossClient.shutdown();
        }
    }

    /**
     * 根据文件扩展名确定Content-Type
     * @param fileName 文件名
     * @return Content-Type
     */
    private String getContentType(String fileName) {
        if (fileName == null) {
            return "application/octet-stream";
        }

        if (fileName.endsWith(".jpg") || fileName.endsWith(".jpeg")) {
            return "image/jpeg";
        } else if (fileName.endsWith(".png")) {
            return "image/png";
        } else if (fileName.endsWith(".gif")) {
            return "image/gif";
        } else if (fileName.endsWith(".txt")) {
            return "text/plain";
        } else if (fileName.endsWith(".pdf")) {
            return "application/pdf";
        } else if (fileName.endsWith(".doc")) {
            return "application/msword";
        } else if (fileName.endsWith(".docx")) {
            return "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
        } else if (fileName.endsWith(".xls")) {
            return "application/vnd.ms-excel";
        } else if (fileName.endsWith(".xlsx")) {
            return "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
        } else if (fileName.endsWith(".html") || fileName.endsWith(".htm")) {
            return "text/html";
        } else if (fileName.endsWith(".css")) {
            return "text/css";
        } else if (fileName.endsWith(".js")) {
            return "application/javascript";
        } else if (fileName.endsWith(".json")) {
            return "application/json";
        } else if (fileName.endsWith(".xml")) {
            return "application/xml";
        } else if (fileName.endsWith(".mp4")) {
            return "video/mp4";
        } else if (fileName.endsWith(".mp3")) {
            return "audio/mpeg";
        } else {
            return "application/octet-stream";
        }
    }
}